@@ -32,9 +32,7 @@ class InputList(object):
 
     def __init__(self):
         """Create an input list"""
-        self.inputs = {}
-        for s in self.semantics:
-            self.inputs[s] = []
+        self.inputs = {s: [] for s in self.semantics}
 
     def addInput(self, offset, semantic, src, set=None):
         """Add an input source to this input list.
@@ -84,20 +82,18 @@ class Source(DaeObject):
     @staticmethod
     def load(collada, localscope, node):
         sourceid = node.get('id')
+        # Try float_array first (most common), pass found node to avoid re-lookup
         arraynode = node.find(collada.tag('float_array'))
         if arraynode is not None:
-            return FloatSource.load(collada, localscope, node)
-
+            return FloatSource.load(collada, localscope, node, arraynode)
         arraynode = node.find(collada.tag('IDREF_array'))
         if arraynode is not None:
-            return IDRefSource.load(collada, localscope, node)
-
+            return IDRefSource.load(collada, localscope, node, arraynode)
         arraynode = node.find(collada.tag('Name_array'))
         if arraynode is not None:
-            return NameSource.load(collada, localscope, node)
+            return NameSource.load(collada, localscope, node, arraynode)
 
-        if arraynode is None:
-            raise DaeIncompleteError('No array found in source %s' % sourceid)
+        raise DaeIncompleteError('No array found in source %s' % sourceid)
 
 
 class FloatSource(Source):
@@ -179,7 +175,7 @@ class FloatSource(Source):
         node.text = txtdata
         node.set('count', str(rawlen))
         node.set('id', self.id + '-array')
-        node = self.xmlnode.find('%s/%s' % (tag('technique_common'), tag('accessor')))
+        node = self.xmlnode.find(f"{tag('technique_common')}/{tag('accessor')}")
         node.clear()
         node.set('count', str(acclen))
         node.set('source', '#' + self.id + '-array')
@@ -189,11 +185,12 @@ class FloatSource(Source):
         self.xmlnode.set('id', self.id)
 
     @staticmethod
-    def load(collada, localscope, node):
+    def load(collada, localscope, node, arraynode=None):
         sourceid = node.get('id')
-        arraynode = node.find(collada.tag('float_array'))
         if arraynode is None:
-            raise DaeIncompleteError('No float_array in source node')
+            arraynode = node.find(collada.tag('float_array'))
+            if arraynode is None:
+                raise DaeIncompleteError('No float_array in source node')
         if arraynode.text is None or arraynode.text.isspace():
             data = numpy.array([], dtype=numpy.float32)
         else:
@@ -201,9 +198,15 @@ class FloatSource(Source):
                 data = numpy.fromstring(arraynode.text, dtype=numpy.float32, sep=' ')
             except ValueError:
                 raise DaeMalformedError('Corrupted float array')
-        data[numpy.isnan(data)] = 0
+            # Replace NaN values with 0
+            data[numpy.isnan(data)] = 0
 
-        paramnodes = node.findall('%s/%s/%s' % (collada.tag('technique_common'), collada.tag('accessor'), collada.tag('param')))
+        # Use cached xpath or build and cache it
+        accessor_path = getattr(collada, '_floatsource_accessor_xpath', None)
+        if accessor_path is None:
+            accessor_path = f"{collada.tag('technique_common')}/{collada.tag('accessor')}/{collada.tag('param')}"
+            collada._floatsource_accessor_xpath = accessor_path
+        paramnodes = node.findall(accessor_path)
         if not paramnodes:
             raise DaeIncompleteError('No accessor info in source node')
         components = [param.get('name') for param in paramnodes]
@@ -296,7 +299,7 @@ class IDRefSource(Source):
         node.text = txtdata
         node.set('count', str(rawlen))
         node.set('id', self.id + '-array')
-        node = self.xmlnode.find('%s/%s' % (tag('technique_common'), tag('accessor')))
+        node = self.xmlnode.find(f"{tag('technique_common')}/{tag('accessor')}")
         node.clear()
         node.set('count', str(acclen))
         node.set('source', '#' + self.id + '-array')
@@ -306,20 +309,26 @@ class IDRefSource(Source):
         self.xmlnode.set('id', self.id)
 
     @staticmethod
-    def load(collada, localscope, node):
+    def load(collada, localscope, node, arraynode=None):
         sourceid = node.get('id')
-        arraynode = node.find(tag('IDREF_array'))
         if arraynode is None:
-            raise DaeIncompleteError('No IDREF_array in source node')
+            arraynode = node.find(collada.tag('IDREF_array'))
+            if arraynode is None:
+                raise DaeIncompleteError('No IDREF_array in source node')
         if arraynode.text is None or arraynode.text.isspace():
             values = []
         else:
             try:
-                values = [v for v in arraynode.text.split()]
+                values = arraynode.text.split()
             except ValueError:
                 raise DaeMalformedError('Corrupted IDREF array')
         data = numpy.array(values, dtype=numpy.str_)
-        paramnodes = node.findall('%s/%s/%s' % (collada.tag('technique_common'), collada.tag('accessor'), collada.tag('param')))
+        # Use cached xpath (same as FloatSource) or build and cache it
+        accessor_path = getattr(collada, '_floatsource_accessor_xpath', None)
+        if accessor_path is None:
+            accessor_path = f"{collada.tag('technique_common')}/{collada.tag('accessor')}/{collada.tag('param')}"
+            collada._floatsource_accessor_xpath = accessor_path
+        paramnodes = node.findall(accessor_path)
         if not paramnodes:
             raise DaeIncompleteError('No accessor info in source node')
         components = [param.get('name') for param in paramnodes]
@@ -403,7 +412,7 @@ class NameSource(Source):
         node.text = txtdata
         node.set('count', str(rawlen))
         node.set('id', self.id + '-array')
-        node = self.xmlnode.find('%s/%s' % (tag('technique_common'), tag('accessor')))
+        node = self.xmlnode.find(f"{tag('technique_common')}/{tag('accessor')}")
         node.clear()
         node.set('count', str(acclen))
         node.set('source', '#' + self.id + '-array')
@@ -413,21 +422,26 @@ class NameSource(Source):
         self.xmlnode.set('id', self.id)
 
     @staticmethod
-    def load(collada, localscope, node):
+    def load(collada, localscope, node, arraynode=None):
         sourceid = node.get('id')
-        arraynode = node.find(tag('Name_array'))
         if arraynode is None:
-            raise DaeIncompleteError('No Name_array in source node')
+            arraynode = node.find(collada.tag('Name_array'))
+            if arraynode is None:
+                raise DaeIncompleteError('No Name_array in source node')
         if arraynode.text is None or arraynode.text.isspace():
             values = []
         else:
             try:
-                values = [v for v in arraynode.text.split()]
+                values = arraynode.text.split()
             except ValueError:
                 raise DaeMalformedError('Corrupted Name array')
         data = numpy.array(values, dtype=numpy.str_)
-        paramnodes = node.findall('%s/%s/%s' % (tag('technique_common'), tag('accessor'), tag
-                                                ('param')))
+        # Use cached xpath (same as FloatSource) or build and cache it
+        accessor_path = getattr(collada, '_floatsource_accessor_xpath', None)
+        if accessor_path is None:
+            accessor_path = f"{collada.tag('technique_common')}/{collada.tag('accessor')}/{collada.tag('param')}"
+            collada._floatsource_accessor_xpath = accessor_path
+        paramnodes = node.findall(accessor_path)
         if not paramnodes:
             raise DaeIncompleteError('No accessor info in source node')
         components = [param.get('name') for param in paramnodes]
